@@ -34,7 +34,11 @@
 #include <common/hathreads.h>
 #include <common/initcall.h>
 
+#ifndef DEBUG_DONT_SHARE_POOLS
 #define MEM_F_SHARED	0x1
+#else
+#define MEM_F_SHARED	0
+#endif
 #define MEM_F_EXACT	0x2
 
 /* reserve an extra void* at the end of a pool for linking */
@@ -46,14 +50,7 @@
 #define POOL_LINK(pool, item) ((void **)(item))
 #endif
 
-/* A special pointer for the pool's free_list that indicates someone is
- * currently manipulating it. Serves as a short-lived lock.
- */
-#define POOL_BUSY ((void *)1)
-
-#ifndef MAX_BASE_POOLS
-#define MAX_BASE_POOLS 64
-#endif
+#define MAX_BASE_POOLS 32
 
 struct pool_cache_head {
 	struct list list;    /* head of objects in this pool */
@@ -70,9 +67,19 @@ extern struct pool_cache_head pool_cache[][MAX_BASE_POOLS];
 extern THREAD_LOCAL size_t pool_cache_bytes;   /* total cache size */
 extern THREAD_LOCAL size_t pool_cache_count;   /* #cache objects   */
 
+#ifdef CONFIG_HAP_LOCKLESS_POOLS
+struct pool_free_list {
+	void **free_list;
+	uintptr_t seq;
+};
+#endif
+
 struct pool_head {
 	void **free_list;
-#ifndef CONFIG_HAP_LOCKLESS_POOLS
+#ifdef CONFIG_HAP_LOCKLESS_POOLS
+	uintptr_t seq;
+	HA_SPINLOCK_T flush_lock;
+#else
 	__decl_hathreads(HA_SPINLOCK_T lock); /* the spin lock */
 #endif
 	unsigned int used;	/* how many chunks are currently in use */
@@ -131,8 +138,8 @@ void create_pool_callback(struct pool_head **ptr, char *name, unsigned int size)
 void dump_pools_to_trash();
 void dump_pools(void);
 int pool_total_failures();
-unsigned long long pool_total_allocated();
-unsigned long long pool_total_used();
+unsigned long pool_total_allocated();
+unsigned long pool_total_used();
 
 /*
  * This function frees whatever can be freed in pool <pool>.
@@ -206,41 +213,31 @@ static inline void *__pool_get_from_cache(struct pool_head *pool)
  */
 static inline void *__pool_get_first(struct pool_head *pool)
 {
+	struct pool_free_list cmp, new;
 	void *ret = __pool_get_from_cache(pool);
 
 	if (ret)
 		return ret;
 
-	/* we'll need to reference the first element to figure the next one. We
-	 * must temporarily lock it so that nobody allocates then releases it,
-	 * or the dereference could fail.
-	 */
-	ret = pool->free_list;
+	cmp.seq = pool->seq;
+	__ha_barrier_load();
+
+	cmp.free_list = pool->free_list;
 	do {
-		while (unlikely(ret == POOL_BUSY)) {
-			pl_cpu_relax();
-			ret = _HA_ATOMIC_LOAD(&pool->free_list);
-		}
-		if (ret == NULL)
-			return ret;
-	} while (unlikely((ret = _HA_ATOMIC_XCHG(&pool->free_list, POOL_BUSY)) == POOL_BUSY));
+		if (cmp.free_list == NULL)
+			return NULL;
+		new.seq = cmp.seq + 1;
+		__ha_barrier_load();
+		new.free_list = *POOL_LINK(pool, cmp.free_list);
+	} while (HA_ATOMIC_DWCAS((void *)&pool->free_list, (void *)&cmp, (void *)&new) == 0);
+	__ha_barrier_atomic_store();
 
-	if (unlikely(ret == NULL)) {
-		_HA_ATOMIC_STORE(&pool->free_list, NULL);
-		goto out;
-	}
-
-	/* this releases the lock */
-	_HA_ATOMIC_STORE(&pool->free_list, *POOL_LINK(pool, ret));
 	_HA_ATOMIC_ADD(&pool->used, 1);
 #ifdef DEBUG_MEMORY_POOLS
 	/* keep track of where the element was allocated from */
-	*POOL_LINK(pool, ret) = (void *)pool;
+	*POOL_LINK(pool, cmp.free_list) = (void *)pool;
 #endif
-
- out:
-	__ha_barrier_atomic_store();
-	return ret;
+	return cmp.free_list;
 }
 
 static inline void *pool_get_first(struct pool_head *pool)
@@ -288,19 +285,14 @@ static inline void *pool_alloc(struct pool_head *pool)
  */
 static inline void __pool_free(struct pool_head *pool, void *ptr)
 {
-	void **free_list;
+	void **free_list = pool->free_list;
 
-	_HA_ATOMIC_SUB(&pool->used, 1);
-	free_list = _HA_ATOMIC_LOAD(&pool->free_list);
 	do {
-		while (unlikely(free_list == POOL_BUSY)) {
-			pl_cpu_relax();
-			free_list = _HA_ATOMIC_LOAD(&pool->free_list);
-		}
-		_HA_ATOMIC_STORE(POOL_LINK(pool, ptr), (void *)free_list);
-		__ha_barrier_atomic_store();
+		*POOL_LINK(pool, ptr) = (void *)free_list;
+		__ha_barrier_store();
 	} while (!_HA_ATOMIC_CAS(&pool->free_list, &free_list, ptr));
 	__ha_barrier_atomic_store();
+	_HA_ATOMIC_SUB(&pool->used, 1);
 }
 
 /* frees an object to the local cache, possibly pushing oldest objects to the
@@ -427,14 +419,27 @@ static inline void pool_free_area(void *area, size_t __maybe_unused size)
 static inline void *pool_alloc_area(size_t size)
 {
 	size_t pad = (4096 - size) & 0xFF0;
+	int isolated;
 	void *ret;
 
+	isolated = thread_isolated();
+	if (!isolated)
+		thread_harmless_now();
 	ret = mmap(NULL, (size + 4095) & -4096, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS, -1, 0);
-	if (ret == MAP_FAILED)
-		return NULL;
-	if (pad >= sizeof(void *))
-		*(void **)(ret + pad - sizeof(void *)) = ret + pad;
-	return ret + pad;
+	if (ret != MAP_FAILED) {
+		/* let's dereference the page before returning so that the real
+		 * allocation in the system is performed without holding the lock.
+		 */
+		*(int *)ret = 0;
+		if (pad >= sizeof(void *))
+			*(void **)(ret + pad - sizeof(void *)) = ret + pad;
+		ret += pad;
+	} else {
+		ret = NULL;
+	}
+	if (!isolated)
+		thread_harmless_end();
+	return ret;
 }
 
 /* frees an area <area> of size <size> allocated by pool_alloc_area(). The
@@ -450,7 +455,9 @@ static inline void pool_free_area(void *area, size_t size)
 	if (pad >= sizeof(void *) && *(void **)(area - sizeof(void *)) != area)
 		*(volatile int *)0 = 0;
 
+	thread_harmless_now();
 	munmap(area - pad, (size + 4095) & -4096);
+	thread_harmless_end();
 }
 
 #endif /* DEBUG_UAF */
@@ -484,7 +491,6 @@ static inline void *pool_alloc(struct pool_head *pool)
 static inline void pool_free(struct pool_head *pool, void *ptr)
 {
         if (likely(ptr != NULL)) {
-		HA_SPIN_LOCK(POOL_LOCK, &pool->lock);
 #ifdef DEBUG_MEMORY_POOLS
 		/* we'll get late corruption if we refill to the wrong pool or double-free */
 		if (*POOL_LINK(pool, ptr) != (void *)pool)
@@ -492,16 +498,20 @@ static inline void pool_free(struct pool_head *pool, void *ptr)
 #endif
 
 #ifndef DEBUG_UAF /* normal pool behaviour */
+		HA_SPIN_LOCK(POOL_LOCK, &pool->lock);
 		*POOL_LINK(pool, ptr) = (void *)pool->free_list;
-                pool->free_list = (void *)ptr;
+		pool->free_list = (void *)ptr;
+		pool->used--;
+		HA_SPIN_UNLOCK(POOL_LOCK, &pool->lock);
 #else  /* release the entry for real to detect use after free */
 		/* ensure we crash on double free or free of a const area*/
 		*(uint32_t *)ptr = 0xDEADADD4;
 		pool_free_area(ptr, pool->size + POOL_EXTRA);
+		HA_SPIN_LOCK(POOL_LOCK, &pool->lock);
 		pool->allocated--;
-#endif /* DEBUG_UAF */
-                pool->used--;
+		pool->used--;
 		HA_SPIN_UNLOCK(POOL_LOCK, &pool->lock);
+#endif /* DEBUG_UAF */
 	}
 }
 #endif /* CONFIG_HAP_LOCKLESS_POOLS */
