@@ -127,8 +127,8 @@ enum srv_initaddr {
     "srvrecord"
 
 #define SRV_STATE_FILE_MAX_FIELDS 20
-#define SRV_STATE_FILE_NB_FIELDS_VERSION_1 20
-#define SRV_STATE_LINE_MAXLEN 2000
+#define SRV_STATE_FILE_NB_FIELDS_VERSION_1 19
+#define SRV_STATE_LINE_MAXLEN 512
 
 /* server flags -- 32 bits */
 #define SRV_F_BACKUP       0x0001        /* this server is a backup server */
@@ -223,7 +223,7 @@ struct server {
 	struct list *priv_conns;		/* private idle connections attached to stream interfaces */
 	struct list *idle_conns;		/* sharable idle connections attached or not to a stream interface */
 	struct list *safe_conns;		/* safe idle connections attached to stream interfaces, shared */
-	struct list *idle_orphan_conns;         /* Orphan connections idling */
+	struct mt_list *idle_orphan_conns;         /* Orphan connections idling */
 	unsigned int pool_purge_delay;          /* Delay before starting to purge the idle conns pool */
 	unsigned int max_idle_conns;            /* Max number of connection allowed in the orphan connections list */
 	unsigned int curr_idle_conns;           /* Current number of orphan idling connections */
@@ -295,7 +295,6 @@ struct server {
 			unsigned char *ptr;
 			int size;
 			int allocated_size;
-			char *sni; /* SNI used for the session */
 		} * reused_sess;
 		char *ciphers;			/* cipher suite to use if non-null */
 #if (HA_OPENSSL_VERSION_NUMBER >= 0x10101000L && !defined OPENSSL_IS_BORINGSSL && !defined LIBRESSL_VERSION_NUMBER)
@@ -346,6 +345,17 @@ struct server {
 	struct sockaddr_storage socks4_addr;	/* the address of the SOCKS4 Proxy, including the port */
 };
 
+
+/* Storage structure to load server-state lines from a flat file into
+ * an ebtree, for faster processing
+ */
+struct state_line {
+	char *line;
+	struct ebmb_node name_name;
+	/* WARNING don't put anything after name_name, it's used by the key */
+};
+
+
 /* Descriptor for a "server" keyword. The ->parse() function returns 0 in case of
  * success, or a combination of ERR_* flags if an error is encountered. The
  * function pointer can be NULL if not implemented. The function also has an
