@@ -313,6 +313,7 @@ cache_store_http_payload(struct stream *s, struct filter *filter, struct http_ms
 	struct htx *htx = htxbuf(&msg->chn->buf);
 	struct htx_blk *blk;
 	struct shared_block *fb;
+	struct htx_ret htxret;
 	unsigned int orig_len, to_forward;
 	int ret;
 
@@ -327,16 +328,15 @@ cache_store_http_payload(struct stream *s, struct filter *filter, struct http_ms
 	chunk_reset(&trash);
 	orig_len = len;
 	to_forward = 0;
-	for (blk = htx_get_first_blk(htx); blk && len; blk = htx_get_next_blk(htx, blk)) {
+
+	htxret = htx_find_offset(htx, offset);
+	blk = htxret.blk;
+	offset = htxret.ret;
+	for (; blk && len; blk = htx_get_next_blk(htx, blk)) {
 		enum htx_blk_type type = htx_get_blk_type(blk);
 		uint32_t info, sz = htx_get_blksz(blk);
 		struct ist v;
 
-		if (offset >= sz) {
-			offset -= sz;
-			continue;
-		}
-
 		switch (type) {
 			case HTX_BLK_UNUSED:
 				break;
@@ -1422,19 +1422,17 @@ parse_cache_flt(char **args, int *cur_arg, struct proxy *px,
 	char *name = NULL;
 	int pos = *cur_arg;
 
-	/* Get the cache filter name*/
-	if (!strcmp(args[pos], "cache")) {
-		if (!*args[pos + 1]) {
-			memprintf(err, "%s : expects an <id> argument", args[pos]);
-			goto error;
-		}
-		name = strdup(args[pos + 1]);
-		if (!name) {
-			memprintf(err, "%s '%s' : out of memory", args[pos], args[pos + 1]);
-			goto error;
-		}
-		pos += 2;
+	/* Get the cache filter name. <pos> point on "cache" keyword */
+	if (!*args[pos + 1]) {
+		memprintf(err, "%s : expects an <id> argument", args[pos]);
+		goto error;
 	}
+	name = strdup(args[pos + 1]);
+	if (!name) {
+		memprintf(err, "%s '%s' : out of memory", args[pos], args[pos + 1]);
+		goto error;
+	}
+	pos += 2;
 
 	/* Check if an implicit filter with the same name already exists. If so,
 	 * we remove the implicit filter to use the explicit one. */
