@@ -40,7 +40,6 @@
 #include <proto/fd.h>
 #include <proto/frontend.h>
 #include <proto/log.h>
-#include <proto/hdr_idx.h>
 #include <proto/mux_pt.h>
 #include <proto/peers.h>
 #include <proto/proxy.h>
@@ -125,48 +124,6 @@ enum {
 	PEER_MSG_ERR_SIZELIMIT,
 };
 
-/* network key types;
- * network types were directly and mistakenly
- * mapped on sample types, to keep backward
- * compatiblitiy we keep those values but
- * we now use a internal/network mapping
- * to avoid further mistakes adding or
- * modifying internals types
- */
-enum {
-        PEER_KT_ANY = 0,  /* any type */
-        PEER_KT_RESV1,    /* UNUSED */
-        PEER_KT_SINT,     /* signed 64bits integer type */
-        PEER_KT_RESV3,    /* UNUSED */
-        PEER_KT_IPV4,     /* ipv4 type */
-        PEER_KT_IPV6,     /* ipv6 type */
-        PEER_KT_STR,      /* char string type */
-        PEER_KT_BIN,      /* buffer type */
-        PEER_KT_TYPES     /* number of types, must always be last */
-};
-
-/* Map used to retrieve network type from internal type
- * Note: Undeclared mapping maps entry to PEER_KT_ANY == 0
- */
-static int peer_net_key_type[SMP_TYPES] = {
-	[SMP_T_SINT] = PEER_KT_SINT,
-	[SMP_T_IPV4] = PEER_KT_IPV4,
-	[SMP_T_IPV6] = PEER_KT_IPV6,
-	[SMP_T_STR]  = PEER_KT_STR,
-	[SMP_T_BIN]  = PEER_KT_BIN,
-};
-
-/* Map used to retrieve internal type from external type
- * Note: Undeclared mapping maps entry to SMP_T_ANY == 0
- */
-static int peer_int_key_type[PEER_KT_TYPES] = {
-	[PEER_KT_SINT] = SMP_T_SINT,
-	[PEER_KT_IPV4] = SMP_T_IPV4,
-	[PEER_KT_IPV6] = SMP_T_IPV6,
-	[PEER_KT_STR]  = SMP_T_STR,
-	[PEER_KT_BIN]  = SMP_T_BIN,
-};
-
 /*
  * Parameters used by functions to build peer protocol messages. */
 struct peer_prep_params {
@@ -662,7 +619,7 @@ static int peer_prepare_switchmsg(char *msg, size_t size, struct peer_prep_param
 
 	/* encode table type */
 
-	intencode(peer_net_key_type[st->table->type], &cursor);
+	intencode(st->table->type, &cursor);
 
 	/* encode table key size */
 	intencode(st->table->key_size, &cursor);
@@ -1697,7 +1654,7 @@ static inline int peer_treat_definemsg(struct appctx *appctx, struct peer *p,
 	if (!*msg_cur)
 		goto malformed_exit;
 
-	if (p->remote_table->table->type != peer_int_key_type[table_type]
+	if (p->remote_table->table->type != table_type
 		|| p->remote_table->table->key_size != table_keylen) {
 		p->remote_table = NULL;
 		goto ignore_msg;
@@ -2545,8 +2502,6 @@ static struct appctx *peer_session_create(struct peers *peers, struct peer *peer
 	struct appctx *appctx;
 	struct session *sess;
 	struct stream *s;
-	struct connection *conn;
-	struct conn_stream *cs;
 
 	peer->reconnect = tick_add(now_ms, MS_TO_TICKS(PEER_RECONNECT_TIMEOUT));
 	peer->heartbeat = tick_add(now_ms, MS_TO_TICKS(PEER_HEARTBEAT_TIMEOUT));
@@ -2571,35 +2526,17 @@ static struct appctx *peer_session_create(struct peers *peers, struct peer *peer
 		goto out_free_sess;
 	}
 
-	/* The tasks below are normally what is supposed to be done by
-	 * fe->accept().
-	 */
-	s->flags = SF_ASSIGNED|SF_ADDR_SET;
-
 	/* applet is waiting for data */
 	si_cant_get(&s->si[0]);
 	appctx_wakeup(appctx);
 
 	/* initiate an outgoing connection */
-	s->si[1].flags |= SI_FL_NOLINGER;
-	si_set_state(&s->si[1], SI_ST_ASS);
-
-	/* automatically prepare the stream interface to connect to the
-	 * pre-initialized connection in si->conn.
-	 */
-	if (unlikely((conn = conn_new()) == NULL))
+	s->target = peer_session_target(peer, s);
+	if (!sockaddr_alloc(&s->target_addr))
 		goto out_free_strm;
-
-	if (unlikely((cs = cs_new(conn)) == NULL))
-		goto out_free_conn;
-
-	conn->target = s->target = peer_session_target(peer, s);
-	memcpy(&conn->addr.to, &peer->addr, sizeof(conn->addr.to));
-
-	conn_prepare(conn, peer->proto, peer_xprt(peer));
-	if (conn_install_mux(conn, &mux_pt_ops, cs, s->be, NULL) < 0)
-		goto out_free_cs;
-	si_attach_cs(&s->si[1], cs);
+	*s->target_addr = peer->addr;
+	s->flags = SF_ASSIGNED|SF_ADDR_SET;
+	s->si[1].flags |= SI_FL_NOLINGER;
 
 	s->do_log = NULL;
 	s->uniq_id = 0;
@@ -2612,10 +2549,6 @@ static struct appctx *peer_session_create(struct peers *peers, struct peer *peer
 	return appctx;
 
 	/* Error unrolling */
-out_free_cs:
-	cs_free(cs);
- out_free_conn:
-	conn_free(conn);
  out_free_strm:
 	LIST_DEL(&s->list);
 	pool_free(pool_head_stream, s);
@@ -3097,12 +3030,8 @@ static int cli_parse_show_peers(char **args, char *payload, struct appctx *appct
 			}
 		}
 
-		if (!p) {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "No such peers\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
-		}
+		if (!p)
+			return cli_err(appctx, "No such peers\n");
 	}
 
 	return 0;
@@ -3187,23 +3116,20 @@ static int peers_dump_peer(struct buffer *msg, struct stream_interface *si, stru
 	if (conn)
 		chunk_appendf(&trash, "\n        xprt=%s", conn_get_xprt_name(conn));
 
-	switch (conn ? addr_to_str(&conn->addr.from, pn, sizeof(pn)) : AF_UNSPEC) {
+	switch (conn && conn_get_src(conn) ? addr_to_str(conn->src, pn, sizeof(pn)) : AF_UNSPEC) {
 	case AF_INET:
 	case AF_INET6:
-		chunk_appendf(&trash, " src=%s:%d", pn, get_host_port(&conn->addr.from));
+		chunk_appendf(&trash, " src=%s:%d", pn, get_host_port(conn->src));
 		break;
 	case AF_UNIX:
 		chunk_appendf(&trash, " src=unix:%d", strm_li(peer_s)->luid);
 		break;
 	}
 
-	if (conn)
-		conn_get_to_addr(conn);
-
-	switch (conn ? addr_to_str(&conn->addr.to, pn, sizeof(pn)) : AF_UNSPEC) {
+	switch (conn && conn_get_dst(conn) ? addr_to_str(conn->dst, pn, sizeof(pn)) : AF_UNSPEC) {
 	case AF_INET:
 	case AF_INET6:
-		chunk_appendf(&trash, " addr=%s:%d", pn, get_host_port(&conn->addr.to));
+		chunk_appendf(&trash, " addr=%s:%d", pn, get_host_port(conn->dst));
 		break;
 	case AF_UNIX:
 		chunk_appendf(&trash, " addr=unix:%d", strm_li(peer_s)->luid);
