@@ -9,7 +9,7 @@
 
 #include <proto/arg.h>
 #include <proto/http_rules.h>
-#include <proto/proto_http.h>
+#include <proto/http_ana.h>
 #include <proto/sample.h>
 #include <proto/stream.h>
 #include <proto/tcp_rules.h>
@@ -271,7 +271,7 @@ static char *register_name(const char *name, int len, enum vars_scope *scope,
 	/* Check variable name syntax. */
 	tmp = var_names[var_names_nb - 1];
 	while (*tmp) {
-		if (!isalnum((unsigned char)*tmp) && *tmp != '_' && *tmp != '.') {
+		if (!isalnum((int)(unsigned char)*tmp) && *tmp != '_' && *tmp != '.') {
 			memprintf(err, "invalid syntax at char '%s'", tmp);
 			res = NULL;
 			goto end;
@@ -571,13 +571,9 @@ void vars_unset_by_name(const char *name, size_t len, struct sample *smp)
 	sample_clear_stream(name, scope, smp);
 }
 
-/* This function fills a sample with the variable content.
- *
- * Keep in mind that a sample content is duplicated by using smp_dup()
- * and it therefore uses a pre-allocated trash chunk as returned by
- * get_trash_chunk().
- *
- * Returns 1 if the sample is filled, otherwise it returns 0.
+/* this function fills a sample with the
+ * variable content. Returns 1 if the sample
+ * is filled, otherwise it returns 0.
  */
 int vars_get_by_name(const char *name, size_t len, struct sample *smp)
 {
@@ -596,29 +592,19 @@ int vars_get_by_name(const char *name, size_t len, struct sample *smp)
 		return 0;
 
 	/* Get the variable entry. */
-	HA_RWLOCK_RDLOCK(VARS_LOCK, &vars->rwlock);
 	var = var_get(vars, name);
-	if (!var) {
-		HA_RWLOCK_RDUNLOCK(VARS_LOCK, &vars->rwlock);
+	if (!var)
 		return 0;
-	}
 
 	/* Copy sample. */
 	smp->data = var->data;
-	smp_dup(smp);
-
-	HA_RWLOCK_RDUNLOCK(VARS_LOCK, &vars->rwlock);
+	smp->flags = SMP_F_CONST;
 	return 1;
 }
 
-/* This function fills a sample with the content of the variable described
- * by <var_desc>.
- *
- * Keep in mind that a sample content is duplicated by using smp_dup()
- * and it therefore uses a pre-allocated trash chunk as returned by
- * get_trash_chunk().
- *
- * Returns 1 if the sample is filled, otherwise it returns 0.
+/* this function fills a sample with the
+ * content of the variable described by <var_desc>. Returns 1
+ * if the sample is filled, otherwise it returns 0.
  */
 int vars_get_by_desc(const struct var_desc *var_desc, struct sample *smp)
 {
@@ -633,18 +619,13 @@ int vars_get_by_desc(const struct var_desc *var_desc, struct sample *smp)
 		return 0;
 
 	/* Get the variable entry. */
-	HA_RWLOCK_RDLOCK(VARS_LOCK, &vars->rwlock);
 	var = var_get(vars, var_desc->name);
-	if (!var) {
-		HA_RWLOCK_RDUNLOCK(VARS_LOCK, &vars->rwlock);
+	if (!var)
 		return 0;
-	}
 
 	/* Copy sample. */
 	smp->data = var->data;
-	smp_dup(smp);
-
-	HA_RWLOCK_RDUNLOCK(VARS_LOCK, &vars->rwlock);
+	smp->flags = SMP_F_CONST;
 	return 1;
 }
 
