@@ -26,7 +26,7 @@
  * It relies on timer_create() and timer_settime() which are only available in
  * this case.
  */
-#if defined(USE_THREAD) && defined(USE_RT) && (_POSIX_TIMERS > 0) && defined(_POSIX_THREAD_CPUTIME)
+#if defined(USE_RT) && (_POSIX_TIMERS > 0) && defined(_POSIX_THREAD_CPUTIME)
 
 /* Setup (or ping) the watchdog timer for thread <thr>. Returns non-zero on
  * success, zero on failure. It interrupts once per second of CPU time. It
@@ -39,7 +39,7 @@ int wdt_ping(int thr)
 
 	its.it_value.tv_sec    = 1; its.it_value.tv_nsec    = 0;
 	its.it_interval.tv_sec = 0; its.it_interval.tv_nsec = 0;
-	return timer_settime(thread_info[thr].wd_timer, 0, &its, NULL) == 0;
+	return timer_settime(ha_thread_info[thr].wd_timer, 0, &its, NULL) == 0;
 }
 
 /* This is the WDTSIG signal handler */
@@ -64,13 +64,11 @@ void wdt_handler(int sig, siginfo_t *si, void *arg)
 		if (thr < 0 || thr >= global.nbthread)
 			break;
 
-		p = thread_info[thr].prev_cpu_time;
-		n = now_cpu_time_thread(&thread_info[thr]);
+		p = ha_thread_info[thr].prev_cpu_time;
+		n = now_cpu_time_thread(&ha_thread_info[thr]);
 
-		/* not yet reached the deadline of 1 sec,
-		 * or p wasn't initialized yet
-		 */
-		if (!p || n - p < 1000000000UL)
+		/* not yet reached the deadline of 1 sec */
+		if (n - p < 1000000000UL)
 			goto update_and_leave;
 
 		if ((threads_harmless_mask|sleeping_thread_mask|threads_to_dump) & (1UL << thr)) {
@@ -92,8 +90,8 @@ void wdt_handler(int sig, siginfo_t *si, void *arg)
 		 * If it's already set, then it's our second call with no
 		 * progress and the thread is dead.
 		 */
-		if (!(thread_info[thr].flags & TI_FL_STUCK)) {
-			_HA_ATOMIC_OR(&thread_info[thr].flags, TI_FL_STUCK);
+		if (!(ha_thread_info[thr].flags & TI_FL_STUCK)) {
+			_HA_ATOMIC_OR(&ha_thread_info[thr].flags, TI_FL_STUCK);
 			goto update_and_leave;
 		}
 
@@ -115,9 +113,11 @@ void wdt_handler(int sig, siginfo_t *si, void *arg)
 	 * with the other thread interrupted exactly where it was running and
 	 * the current one not involved in this.
 	 */
+#ifdef USE_THREAD
 	if (thr != tid)
-		pthread_kill(thread_info[thr].pthread, sig);
+		pthread_kill(ha_thread_info[thr].pthread, sig);
 	else
+#endif
 		ha_panic();
 	return;
 
